@@ -24,6 +24,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdint.h>
+#include <math.h>
 #include "main.h"
 #include "input.h"
 #include "callbacks.h"
@@ -122,7 +123,11 @@ void on_monitors_changed ( GdkScreen *screen,
   cairo_destroy (cr);
   cairo_surface_destroy(data->backbuffer);
   data->backbuffer = new_shape;
- 
+
+  // recreate auxiliary backbuffer
+  cairo_surface_destroy(data->aux_backbuffer);
+  data->aux_backbuffer = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, data->width, data->height);
+
   /*
      these depend on the shape surface
   */
@@ -137,10 +142,10 @@ void on_monitors_changed ( GdkScreen *screen,
   parse_config(data); // also calls paint_context_new() :-(
 
 
-  data->default_pen = paint_context_new (data, GROMIT_PEN,
-					 data->red, 7, 0, 1, G_MAXUINT);
-  data->default_eraser = paint_context_new (data, GROMIT_ERASER,
-					    data->red, 75, 0, 1, G_MAXUINT);
+  data->default_pen = paint_context_new (data, GROMIT_PEN, data->red, 7,
+                                         0, GROMIT_ARROW_END, 1, G_MAXUINT);
+  data->default_eraser = paint_context_new (data, GROMIT_ERASER, data->red, 75,
+                                            0, GROMIT_ARROW_END, 1, G_MAXUINT);
 
   if(!data->composited) // set shape
     {
@@ -273,6 +278,14 @@ gboolean on_buttonpress (GtkWidget *win,
       devdata->lastslave != gdk_event_get_source_device ((GdkEvent *) ev))
     select_tool (data, ev->device, gdk_event_get_source_device ((GdkEvent *) ev), ev->state);
 
+  GromitPaintType type = devdata->cur_context->type;
+
+  // store original state to have dynamic update of line and rect
+  if (type == GROMIT_LINE || type == GROMIT_RECT)
+    {
+      copy_surface(data->aux_backbuffer, data->backbuffer);
+    }
+
   devdata->lastx = ev->x;
   devdata->lasty = ev->y;
   devdata->motion_time = ev->time;
@@ -319,38 +332,43 @@ gboolean on_motion (GtkWidget *win,
       devdata->lastslave != gdk_event_get_source_device ((GdkEvent *) ev))
     select_tool (data, ev->device, gdk_event_get_source_device ((GdkEvent *) ev), ev->state);
 
+  GromitPaintType type = devdata->cur_context->type;
+
   gdk_device_get_history (ev->device, ev->window,
 			  devdata->motion_time, ev->time,
 			  &coords, &nevents);
 
   if(!data->xinerama && nevents > 0)
     {
-      for (i=0; i < nevents; i++)
+      if (type != GROMIT_LINE && type != GROMIT_RECT)
         {
-          gdouble x, y;
-
-          gdk_device_get_axis (ev->device, coords[i]->axes,
-                               GDK_AXIS_PRESSURE, &pressure);
-          if (pressure > 0)
+          for (i=0; i < nevents; i++)
             {
-	      data->maxwidth = (CLAMP (pressure + line_thickener, 0, 1) *
-				(double) (devdata->cur_context->width -
-					  devdata->cur_context->minwidth) +
-				devdata->cur_context->minwidth);
+              gdouble x, y;
 
-	      if(data->maxwidth > devdata->cur_context->maxwidth)
-		data->maxwidth = devdata->cur_context->maxwidth;
+              gdk_device_get_axis (ev->device, coords[i]->axes,
+                                   GDK_AXIS_PRESSURE, &pressure);
+              if (pressure > 0)
+                {
+                  data->maxwidth = (CLAMP (pressure + line_thickener, 0, 1) *
+                                    (double) (devdata->cur_context->width -
+                                              devdata->cur_context->minwidth) +
+                                    devdata->cur_context->minwidth);
 
-              gdk_device_get_axis(ev->device, coords[i]->axes,
-                                  GDK_AXIS_X, &x);
-              gdk_device_get_axis(ev->device, coords[i]->axes,
-                                  GDK_AXIS_Y, &y);
+                  if(data->maxwidth > devdata->cur_context->maxwidth)
+                    data->maxwidth = devdata->cur_context->maxwidth;
 
-	      draw_line (data, ev->device, devdata->lastx, devdata->lasty, x, y);
+                  gdk_device_get_axis(ev->device, coords[i]->axes,
+                                      GDK_AXIS_X, &x);
+                  gdk_device_get_axis(ev->device, coords[i]->axes,
+                                      GDK_AXIS_Y, &y);
 
-              coord_list_prepend (data, ev->device, x, y, data->maxwidth);
-              devdata->lastx = x;
-              devdata->lasty = y;
+                  draw_line (data, ev->device, devdata->lastx, devdata->lasty, x, y);
+
+                  coord_list_prepend (data, ev->device, x, y, data->maxwidth);
+                  devdata->lastx = x;
+                  devdata->lasty = y;
+                }
             }
         }
 
@@ -373,13 +391,46 @@ gboolean on_motion (GtkWidget *win,
 
       if(devdata->motion_time > 0)
 	{
-	  draw_line (data, ev->device, devdata->lastx, devdata->lasty, ev->x, ev->y);
-	  coord_list_prepend (data, ev->device, ev->x, ev->y, data->maxwidth);
+          if (type == GROMIT_LINE || type == GROMIT_RECT) {
+            copy_surface(data->backbuffer, data->aux_backbuffer);
+            GdkRectangle rect = {0, 0, data->width, data->height};
+            gdk_window_invalidate_rect(gtk_widget_get_window(data->win), &rect, 0);
+          }
+          if (type == GROMIT_LINE)
+            {
+	      draw_line (data, ev->device, devdata->lastx, devdata->lasty, ev->x, ev->y);
+              if (devdata->cur_context->arrowsize > 0)
+                {
+                  GromitArrowType atype = devdata->cur_context->arrow_type;
+                  gint width = devdata->cur_context->arrowsize * devdata->cur_context->width / 2;
+                  gfloat direction =
+                      atan2(ev->y - devdata->lasty, ev->x - devdata->lastx);
+                  if (atype & GROMIT_ARROW_END)
+                    draw_arrow(data, ev->device, ev->x, ev->y, width * 2, direction);
+                  if (atype & GROMIT_ARROW_START)
+                    draw_arrow(data, ev->device, devdata->lastx, devdata->lasty, width * 2, M_PI + direction);
+                }
+            }
+          else if (type == GROMIT_RECT)
+            {
+              draw_line (data, ev->device, devdata->lastx, devdata->lasty, ev->x, devdata->lasty);
+              draw_line (data, ev->device, ev->x, devdata->lasty, ev->x, ev->y);
+              draw_line (data, ev->device, ev->x, ev->y, devdata->lastx, ev->y);
+              draw_line (data, ev->device, devdata->lastx, ev->y, devdata->lastx, devdata->lasty);
+            }
+          else
+            {
+              draw_line (data, ev->device, devdata->lastx, devdata->lasty, ev->x, ev->y);
+	      coord_list_prepend (data, ev->device, ev->x, ev->y, data->maxwidth);
+            }
 	}
     }
 
-  devdata->lastx = ev->x;
-  devdata->lasty = ev->y;
+  if (type != GROMIT_LINE && type != GROMIT_RECT)
+    {
+      devdata->lastx = ev->x;
+      devdata->lasty = ev->y;
+    }
   devdata->motion_time = ev->time;
 
   return TRUE;
@@ -406,11 +457,34 @@ gboolean on_buttonrelease (GtkWidget *win,
 
   if (!devdata->is_grabbed)
     return FALSE;
-  
-  if (devdata->cur_context->arrowsize != 0 &&
-      coord_list_get_arrow_param (data, ev->device, width * 3,
-				  &width, &direction))
-    draw_arrow (data, ev->device, ev->x, ev->y, width, direction);
+
+  GromitPaintType type = devdata->cur_context->type;
+
+  if (devdata->cur_context->arrowsize != 0)
+    {
+      GromitArrowType atype = devdata->cur_context->arrow_type;
+      if (type == GROMIT_LINE)
+        {
+          direction = atan2 (ev->y - devdata->lasty, ev->x - devdata->lastx);
+          if (atype & GROMIT_ARROW_END)
+            draw_arrow(data, ev->device, ev->x, ev->y, width * 2, direction);
+          if (atype & GROMIT_ARROW_START)
+            draw_arrow(data, ev->device, devdata->lastx, devdata->lasty, width * 2, M_PI + direction);
+        }
+      else
+        {
+          gint x0, y0;
+          if ((atype & GROMIT_ARROW_END) &&
+              coord_list_get_arrow_param (data, ev->device, width * 3,
+                                          GROMIT_ARROW_END, &x0, &y0, &width, &direction))
+            draw_arrow (data, ev->device, x0, y0, width, direction);
+          if ((atype & GROMIT_ARROW_START) &&
+              coord_list_get_arrow_param (data, ev->device, width * 3,
+                                          GROMIT_ARROW_START, &x0, &y0, &width, &direction)) {
+            draw_arrow (data, ev->device, x0, y0, width, direction);
+          }
+        }
+    }
 
   coord_list_free (data, ev->device);
 
@@ -540,7 +614,9 @@ void on_mainapp_selection_received (GtkWidget *widget,
 	      g_printerr ("Unable to parse color. "
 	      "Keeping default.\n");
 	    }
-	  GromitPaintContext* line_ctx = paint_context_new(data, GROMIT_PEN, fg_color, thickness, 0, thickness, thickness);
+	  GromitPaintContext* line_ctx =
+            paint_context_new(data, GROMIT_PEN, fg_color, thickness,
+                              0, GROMIT_ARROW_END, thickness, thickness);
 
 	  GdkRectangle rect;
 	  rect.x = MIN (startX,endX) - thickness / 2;
@@ -575,7 +651,7 @@ void on_device_removed (GdkDeviceManager *device_manager,
 {
   GromitData *data = (GromitData *) user_data;
     
-  if(!gdk_device_get_device_type(device) == GDK_DEVICE_TYPE_MASTER
+  if(gdk_device_get_device_type(device) != GDK_DEVICE_TYPE_MASTER
      || gdk_device_get_n_axes(device) < 2)
     return;
   
@@ -591,7 +667,7 @@ void on_device_added (GdkDeviceManager *device_manager,
 {
   GromitData *data = (GromitData *) user_data;
 
-  if(!gdk_device_get_device_type(device) == GDK_DEVICE_TYPE_MASTER
+  if(gdk_device_get_device_type(device) != GDK_DEVICE_TYPE_MASTER
      || gdk_device_get_n_axes(device) < 2)
     return;
 
@@ -732,6 +808,7 @@ void on_about(GtkMenuItem *menuitem,
 				"avma <avi.markovitz@gmail.com>",
 				"godmar <godmar@gmail.com>",
 				"Ashwin Rajesh <46510831+VanillaViking@users.noreply.github.com>",
+                                "Pascal Niklaus <pascal.niklaus@ieu.uzh.ch>",
 				NULL };
     gtk_show_about_dialog (NULL,
 			   "program-name", "Gromit-MPX",
@@ -741,7 +818,7 @@ void on_about(GtkMenuItem *menuitem,
 			   "version", PACKAGE_VERSION,
 			   "website", PACKAGE_URL,
 			   "authors", authors,
-			   "copyright", "2009-2023 Christian Beier, Copyright 2000 Simon Budig",
+			   "copyright", "2009-2024 Christian Beier, Copyright 2000 Simon Budig",
 			   "license-type", GTK_LICENSE_GPL_2_0,
 			   NULL);
 }
@@ -824,6 +901,16 @@ void on_intro(GtkMenuItem *menuitem,
     gtk_widget_show_all (assistant);
 }
 
+void on_issues(GtkMenuItem *menuitem,
+               gpointer user_data)
+{
+    gtk_show_uri_on_window (NULL,
+			    "https://github.com/bk138/gromit-mpx/issues",
+			    GDK_CURRENT_TIME,
+			    NULL);
+}
+
+
 void on_support_liberapay(GtkMenuItem *menuitem, gpointer user_data)
 {
     gtk_show_uri_on_window (NULL,
